package com.example.demo;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;
import org.springframework.http.MediaType;
import org.springframework.web.reactive.function.client.WebClient;
import org.springframework.web.reactive.function.client.support.WebClientAdapter;
import org.springframework.web.service.annotation.GetExchange;
import org.springframework.web.service.invoker.HttpServiceProxyFactory;
import org.springframework.web.servlet.function.RouterFunction;
import org.springframework.web.servlet.function.ServerResponse;

import java.util.List;
import java.util.Map;

import static org.springframework.web.servlet.function.RequestPredicates.GET;
import static org.springframework.web.servlet.function.RouterFunctions.*;

@SpringBootApplication
public class DemoApplication {

	public static void main(String[] args) {
		SpringApplication.run(DemoApplication.class, args);
	}

	@Bean
	public RouterFunction<ServerResponse> testRouterFunction(TodoClient todoClient) {
		return route(GET("/test"), request -> {
			var todos = todoClient.getTodos();
			return ServerResponse.ok().contentType(MediaType.APPLICATION_JSON).body(todos);
		});
	}

	@Bean
	public TodoClient todoClient(WebClient.Builder webClientBuilder, @Value("${todos-url}") String todosUrl) {
		var webClient = webClientBuilder.baseUrl(todosUrl).build();
		return HttpServiceProxyFactory.builder(WebClientAdapter.forClient(webClient)).build().createClient(TodoClient.class);
	}

	public interface TodoClient {

		@GetExchange(url = "/todos", accept = MediaType.APPLICATION_JSON_VALUE)
		List<Map<String, Object>> getTodos();

	}

}
